// 搜索mixin
export const searchUserMixin = {
  data () {
    return {
      isSearch: false, // 搜索展示
      searchList: null // 搜索结果
    }
  },
  methods: {
    searchUser (e, originList) {
      let val = e.target.value
      if (!val) {
        this.isSearch = false
        return
      }

      this.searchList = originList.filter(item => {
        let name = item.name || item.nick_name
        return name.indexOf(val) > -1
      })
      this.isSearch = true
    }
  }
}

// @人
export const chatAtMixin = {
  data () {
    return {
      atInd: 0 // @人索引
    }
  },
  methods: {
    atPerson (name) {
      let el = this.$refs.chatInput
      let selectionStart = el.selectionStart
      let realStart = selectionStart - this.keyAfterAt.length
      this.inputMsg = this.inputMsg.slice(0, realStart) + `${name} ` + this.inputMsg.slice(selectionStart)
      this.atInd = 0
      el.focus()
      this.$nextTick(() => {
        el.setSelectionRange(realStart + name.length + 1, realStart + name.length + 1)
      })
    },
    handleUp () {
      if (this.atInd > 0) {
        this.atInd--
      }
    },
    handleDown () {
      let membersLen = this.filterMembers.length
      this.atInd < membersLen - 1 ? this.atInd++ : this.atInd = 0
    }
  }
}

// 聊天输入框
export const chatInputMixin = {
  data () {
    return {
      selectionAfterAt: false, // 光标在@后面
      keyAfterAt: '' // 光标后的搜索关键字
    }
  },
  computed: {
    filterMembers () {
      let members = this.group.members
      let resArr = []
      if (this.keyAfterAt !== '') {
        for (let k in members) {
          let userName = members[k].user_name
          let nickName = members[k].nick_name
          if (k == this.userId) continue
          if ((nickName.indexOf(this.keyAfterAt) !== -1 || userName.indexOf(this.keyAfterAt) !== -1) && this.inputMsg.indexOf('@' + userName) === -1) {
            resArr.push(members[k])
          }
        }
      } else {
        for (let k in members) {
          if (k == this.userId) continue
          if (this.inputMsg.indexOf('@' + members[k].user_name) === -1) {
            resArr.push(members[k])
          }
        }
      }
      return resArr
    },
    atShow () {
      this.atInd = 0
      return this.selectionAfterAt && this.filterMembers.length
    }
  },
  watch: {
    inputMsg (val, newval) {
      // this.fixIOS()

      this.handleSelectionChange()
    }
  },
  methods: {
    getStrBeforeSelection () {
      let el = this.$refs.chatInput
      if (!el) return ''
      let selectionStart = el.selectionStart
      let prevStr = this.inputMsg.slice(0, selectionStart)
      return prevStr
    },
    getStrAfterSelection () {
      let el = this.$refs.chatInput
      if (!el) return ''
      let selectionStart = el.selectionStart
      let prevStr = this.inputMsg.slice(selectionStart)
      return prevStr
    },
    handleLeft (event) {
      let el = this.$refs.chatInput
      let selectionStart = el.selectionStart
      if (selectionStart === 0) return
      let prevStr = this.getStrBeforeSelection()
      let members = this.group.members

      for (let k in members) {
        let name = members[k].user_name
        let reg = new RegExp(`@${name} $`)
        if (reg.test(prevStr)) {
          event.preventDefault()
          el.setSelectionRange(selectionStart - name.length - 2, selectionStart - name.length - 2)
          return
        }
      }
    },
    handleRight (event) {
      let el = this.$refs.chatInput
      let selectionStart = el.selectionStart
      let afterStr = this.getStrAfterSelection()
      let members = this.group.members

      for (let k in members) {
        let name = members[k].user_name
        let reg = new RegExp(`^@${name} `)
        if (reg.test(afterStr)) {
          event.preventDefault()
          el.setSelectionRange(selectionStart + name.length + 2, selectionStart + name.length + 2)
          return
        }
      }
    },
    handleDel (event) {
      let el = this.$refs.chatInput
      let selectionStart = el.selectionStart
      if (selectionStart === 0) return
      let prevStr = this.getStrBeforeSelection()
      let members = this.group.members

      for (let k in members) {
        let name = members[k].user_name
        let reg = new RegExp(`@${name} $`)
        if (reg.test(prevStr)) {
          event.preventDefault()
          this.inputMsg = this.inputMsg.slice(0, selectionStart - name.length - 2) + this.inputMsg.slice(selectionStart)
          this.$nextTick(() => {
            el.setSelectionRange(selectionStart - name.length - 2, selectionStart - name.length - 2)
          })
          return
        }
      }
    },
    handleKeyDown (event) {
      if (this.atShow) {
        event.preventDefault()
        let item = this.filterMembers[this.atInd]
        this.atPerson(item.user_name)
        return
      }

      if (event.altKey || event.ctrlKey) {
        // 单纯换行
        this.inputMsg = this.inputMsg + '\n'
      } else {
        event.returnValue = false
        this.handleSend(event)
      }

      return false
    },
    handleFocus () {
      // this.fixIOS()

      document.addEventListener('selectionchange', this.handleSelectionChange)
    },
    handleBlur () {
      // this.fixIOS(false)

      document.removeEventListener('selectionchange', this.handleSelectionChange)
      this.updateChatInputFocus(false)
    },
    /**
     * 监听光标位置
     */
    handleSelectionChange () {
      let el = this.$refs.chatInput
      if (!el) return
      let selectionStart = el.selectionStart
      let selectionEnd = this.$refs.chatInput.selectionEnd
      if (selectionStart !== selectionEnd) return

      let prevStr = this.getStrBeforeSelection()
      this.selectionAfterAt = /@/.test(prevStr)
      if (this.selectionAfterAt) {
        this.keyAfterAt = prevStr.slice(prevStr.lastIndexOf('@') + 1)
      }
    },
    /**
     * @des 处理ios系统下窗体闪烁的bug
     * @param {boolean} flag {true: 激活修复状态, false: 恢复原始状态}
     */
    fixIOS (flag = true) {
      let el = this.$refs.scrollWrap
      if (!el) return
      if (flag) {
        el.style.overflowY = 'hidden'
      } else {
        el.overflowY = 'scroll'
      }
    }
  }
}
